<?php

namespace Assetise\Asset\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Assetise\Asset\Forms\Expenditure as ExpenditureForm;
use Assetise\Asset\Http\Requests\ExpenditureResourceRequest;
use Assetise\Asset\Http\Resources\ExpenditureResource;
use Assetise\Asset\Http\Resources\ExpendituresCollection;
use Assetise\Asset\Models\Expenditure;
use Assetise\Asset\Actions\ExpenditureAction;
use Assetise\Asset\Actions\ExpenditureActions;


/**
 * Resource controller class for expenditure.
 */
class ExpenditureResourceController extends BaseController
{

    /**
     * Initialize expenditure resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = ExpenditureForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('assetise.asset.modules'), 'asset', guard_url('asset'));
    }

    /**
     * Display a list of expenditure.
     *
     * @return Response
     */
    public function index(ExpenditureResourceRequest $request)
    {
        $request = $request->all();
        $page = ExpenditureActions::run('paginate', $request);

        $data = new ExpendituresCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('asset::expenditure.names'))
            ->view('asset::expenditure.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display expenditure.
     *
     * @param Request $request
     * @param Model   $expenditure
     *
     * @return Response
     */
    public function show(ExpenditureResourceRequest $request, Expenditure $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExpenditureResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('asset::expenditure.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('asset::expenditure.show')
            ->output();
    }

    /**
     * Show the form for creating a new expenditure.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ExpenditureResourceRequest $request, Expenditure $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExpenditureResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('asset::expenditure.name'))
            ->view('asset::expenditure.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new expenditure.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ExpenditureResourceRequest $request, Expenditure $model)
    {
        try {
            $request = $request->all();
            $model = ExpenditureAction::run('store', $model, $request);
            $data = new ExpenditureResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('asset::expenditure.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('asset/expenditure/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/asset/expenditure'))
                ->redirect();
        }

    }

    /**
     * Show expenditure for editing.
     *
     * @param Request $request
     * @param Model   $expenditure
     *
     * @return Response
     */
    public function edit(ExpenditureResourceRequest $request, Expenditure $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExpenditureResource($model);
        // return view('asset::expenditure.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('asset::expenditure.name'))
            ->view('asset::expenditure.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the expenditure.
     *
     * @param Request $request
     * @param Model   $expenditure
     *
     * @return Response
     */
    public function update(ExpenditureResourceRequest $request, Expenditure $model)
    {
        try {
            $request = $request->all();
            $model = ExpenditureAction::run('update', $model, $request);
            $data = new ExpenditureResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('asset::expenditure.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('asset/expenditure/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/expenditure/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the expenditure.
     *
     * @param Model   $expenditure
     *
     * @return Response
     */
    public function destroy(ExpenditureResourceRequest $request, Expenditure $model)
    {
        try {

            $request = $request->all();
            $model = ExpenditureAction::run('destroy', $model, $request);
            $data = new ExpenditureResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('asset::expenditure.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('asset/expenditure/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/expenditure/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
