<?php

namespace Assetise\Asset\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Assetise\Asset\Forms\Inspection as InspectionForm;
use Assetise\Asset\Http\Requests\InspectionResourceRequest;
use Assetise\Asset\Http\Resources\InspectionResource;
use Assetise\Asset\Http\Resources\InspectionsCollection;
use Assetise\Asset\Models\Inspection;
use Assetise\Asset\Actions\InspectionAction;
use Assetise\Asset\Actions\InspectionActions;


/**
 * Resource controller class for inspection.
 */
class InspectionResourceController extends BaseController
{

    /**
     * Initialize inspection resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = InspectionForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('assetise.asset.modules'), 'asset', guard_url('asset'));
    }

    /**
     * Display a list of inspection.
     *
     * @return Response
     */
    public function index(InspectionResourceRequest $request)
    {
        $request = $request->all();
        $page = InspectionActions::run('paginate', $request);

        $data = new InspectionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('asset::inspection.names'))
            ->view('asset::inspection.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display inspection.
     *
     * @param Request $request
     * @param Model   $inspection
     *
     * @return Response
     */
    public function show(InspectionResourceRequest $request, Inspection $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InspectionResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('asset::inspection.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('asset::inspection.show')
            ->output();
    }

    /**
     * Show the form for creating a new inspection.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InspectionResourceRequest $request, Inspection $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InspectionResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('asset::inspection.name'))
            ->view('asset::inspection.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new inspection.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InspectionResourceRequest $request, Inspection $model)
    {
        try {
            $request = $request->all();
            $model = InspectionAction::run('store', $model, $request);
            $data = new InspectionResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('asset::inspection.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('asset/inspection/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/asset/inspection'))
                ->redirect();
        }

    }

    /**
     * Show inspection for editing.
     *
     * @param Request $request
     * @param Model   $inspection
     *
     * @return Response
     */
    public function edit(InspectionResourceRequest $request, Inspection $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new InspectionResource($model);
        // return view('asset::inspection.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('asset::inspection.name'))
            ->view('asset::inspection.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the inspection.
     *
     * @param Request $request
     * @param Model   $inspection
     *
     * @return Response
     */
    public function update(InspectionResourceRequest $request, Inspection $model)
    {
        try {
            $request = $request->all();
            $model = InspectionAction::run('update', $model, $request);
            $data = new InspectionResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('asset::inspection.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('asset/inspection/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/inspection/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the inspection.
     *
     * @param Model   $inspection
     *
     * @return Response
     */
    public function destroy(InspectionResourceRequest $request, Inspection $model)
    {
        try {

            $request = $request->all();
            $model = InspectionAction::run('destroy', $model, $request);
            $data = new InspectionResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('asset::inspection.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('asset/inspection/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/inspection/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
